package shapecalculus;

import java.util.List;

import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.grid.GridPoint;
import shapecalculus.behaviour.ActionDispatcher;
import shapecalculus.physics.Motion;

public class ComplexShape implements Shape {

	private List<BasicShape> shapes;
	private double mass;
	private NdPoint velocity;
	private GridPoint discreteVelocity;

	public ComplexShape(List<BasicShape> shapes) {
		this.shapes = shapes;
		this.mass = 0;
		for (BasicShape s : this.shapes) {
			s.setComplexShape(this);
			this.mass += s.getMass();
		}
	}

	public List<BasicShape> getShapes() {
		return shapes;
	}

	@Override
	// vige la quantit di moto
	public void updateVelocity() {
		NdPoint qTotal = new NdPoint(0, 0, 0);
		NdPoint currentVel;
		double currentMass;
		for (BasicShape s : shapes) {
			currentVel = s.getVelocity();
			currentMass = s.getMass();
			qTotal = Utils.sumPoints(
					qTotal,
					new NdPoint(currentVel.getX() * currentMass, currentVel
							.getY() * currentMass));
		}
		NdPoint velTotal = new NdPoint(qTotal.getX() / this.mass, qTotal.getY()
				/ this.mass);
		GridPoint discreteVel = Utils.discretize(velTotal);
		this.setVelocity(velTotal);
		this.setDiscreteVelocity(discreteVel);
		for (BasicShape s : shapes)
			s.move();
	}

	@Override
	public void setMotion(Motion law) {
		for (Shape s : shapes)
			s.setMotion(law);
	}

	@Override
	public void addMotion(Motion law) {
		for (Shape s : shapes)
			s.addMotion(law);
	}

	@Override
	public void applyForce(NdPoint forceVector) {
		final NdPoint acceleration = new NdPoint(
				forceVector.getX() / this.mass, forceVector.getY() / this.mass,
				forceVector.getZ() / this.mass);
		Motion forceMotion = new Motion() {
			@Override
			public NdPoint calculateVelocity(Shape s) {
				return Utils.sumPoints(s.getVelocity(), acceleration);
			}
		};
		this.addMotion(forceMotion);

	}

	@Override
	public Motion getMotion() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public double getMass() {
		return this.mass;
	}

	@Override
	public GridPoint getDiscretePosition() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public NdPoint getPosition() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void setDiscretePosition(GridPoint discretePosition) {
		// TODO Auto-generated method stub

	}

	@Override
	public void setPosition(NdPoint position) {
		// TODO Auto-generated method stub

	}

	@Override
	public GridPoint getDiscreteVelocity() {
		return this.discreteVelocity;
	}

	@Override
	public NdPoint getVelocity() {
		return this.velocity;
	}

	@Override
	public void setDiscreteVelocity(GridPoint discreteVelocity) {
		this.discreteVelocity = discreteVelocity;
		for (BasicShape s : shapes)
			s.setDiscreteVelocity(discreteVelocity);
	}

	@Override
	public void setVelocity(NdPoint velocity) {
		this.velocity = velocity;
		for (BasicShape s : shapes)
			s.setVelocity(velocity);
	}

	public boolean moveTo(NdPoint delta) {
		boolean moved = false;
		for (BasicShape s : shapes)
			moved = s.moveTo(Utils.sumPoints(s.getPosition(), delta),
					true) || moved;
		return moved;
	}

	@Override
	public void dispatchEnabledActions(ActionDispatcher dispatcher) {
		for (BasicShape s : shapes)
			s.dispatchEnabledActions(dispatcher);
	}
}
