package shapecalculus;

import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.grid.GridPoint;

public class Surface {

	public static final int SOUTH = 0;
	public static final int NORTH = 1;
	public static final int WEST = 2;
	public static final int EAST = 3;
	public static final int OUTSIDE = 4;
	public static final int INSIDE = 5;
	public static final int[] ALL = new int[] { 0, 1, 2, 3, 4, 5 };

	private int directions[];

	public Surface(int... directions) throws ShapeException {
		this.directions = new int[directions.length];
		if (directions.length < 1)
			throw new ShapeException("Invalid number of surfaces.");
		for (int i = 0; i < directions.length; i++) {
			int direction = directions[i];
			if (direction < 0 || direction > 5)
				throw new ShapeException(
						"Not valid surface index. Try with Surface.(NORTH|SOUTH|WEST|EAST|OUTSIDE|INSIDE)");
			this.directions[i] = direction;
		}
	}

	public int[] getDirection() {
		return directions;
	}

	public boolean isOppositeTo(Surface surface) {
		switch (surface.directions[0]) {
		case SOUTH:
			return this.directions[0] == NORTH;
		case NORTH:
			return this.directions[0] == SOUTH;
		case EAST:
			return this.directions[0] == WEST;
		case WEST:
			return this.directions[0] == EAST;
		case INSIDE:
			return this.directions[0] == OUTSIDE;
		case OUTSIDE:
			return this.directions[0] == INSIDE;

		default:
			break;
		}
		return false;
	}

	public static Surface getOppositeTo(Surface surface) throws ShapeException {
		switch (surface.directions[0]) {
		case SOUTH:
			return new Surface(NORTH);
		case NORTH:
			return new Surface(SOUTH);
		case EAST:
			return new Surface(WEST);
		case WEST:
			return new Surface(EAST);
		case INSIDE:
			return new Surface(OUTSIDE);
		case OUTSIDE:
			return new Surface(INSIDE);

		default:
			break;
		}
		return null;
	}

	public NdPoint getAdjacentPoint(NdPoint currentPoint) {
		NdPoint inc = new NdPoint(0, 0, 0);
		switch (this.directions[0]) {
		case SOUTH:
			inc = new NdPoint(0, -1, 0);
		case NORTH:
			inc = new NdPoint(0, 1, 0);
		case EAST:
			inc = new NdPoint(0, 1, 0);
		case WEST:
			inc = new NdPoint(0, -1, 0);
		case INSIDE:
			inc = new NdPoint(0, 1, 0);
		case OUTSIDE:
			inc = new NdPoint(0, -1, 0);
		default:
			break;
		}
		return Utils.sumPoints(currentPoint, inc);
	}
	
	public GridPoint getAdjacentPoint(GridPoint currentPoint) {
		GridPoint inc = new GridPoint(0, 0, 0);
		switch (this.directions[0]) {
		case SOUTH:
			inc = new GridPoint(0, -1, 0);
		case NORTH:
			inc = new GridPoint(0, 1, 0);
		case EAST:
			inc = new GridPoint(0, 1, 0);
		case WEST:
			inc = new GridPoint(0, -1, 0);
		case INSIDE:
			inc = new GridPoint(0, 1, 0);
		case OUTSIDE:
			inc = new GridPoint(0, -1, 0);
		default:
			break;
		}
		return Utils.sumPoints(currentPoint, inc);
	}

}
