package shapecalculus;

import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.grid.GridPoint;

public class Utils {

	public static GridPoint discretize(NdPoint realPoint) {
		int[] coords = new int[realPoint.dimensionCount()];
		for (int i = 0; i < coords.length; i++)
			coords[i] = (int) realPoint.getCoord(i);
		return new GridPoint(coords);
	}

	public static NdPoint continuize(GridPoint discretePoint) {
		double[] coords = new double[discretePoint.dimensionCount()];
		for (int i = 0; i < coords.length; i++)
			coords[i] = discretePoint.getCoord(i);
		return new NdPoint(coords);
	}

	public static GridPoint abs(GridPoint point) {
		return new GridPoint(Math.abs(point.getX()), point.getY());
	}

	public static NdPoint abs(NdPoint point) {
		return new NdPoint(Math.abs(point.getX()), point.getY());
	}

	public static GridPoint max(GridPoint... points) {
		int x = Integer.MIN_VALUE, y = Integer.MIN_VALUE;
		for (GridPoint point : points) {

			x = Math.max(x, point.getX());
			y = Math.max(y, point.getY());
			// z = Math.max(z, point.getZ());
		}
		return new GridPoint(x, y);
	}

	public static NdPoint max(NdPoint... points) {
		double x = Double.MIN_VALUE, y = Double.MIN_VALUE;
		for (NdPoint point : points) {
			x = Math.max(x, point.getX());
			y = Math.max(y, point.getY());
			// z = Math.max(z, point.getZ());
		}
		return new NdPoint(x, y);
	}

	public static NdPoint negPoint(NdPoint point) {
		return new NdPoint(-point.getX(), -point.getY());
	}

	public static GridPoint negPoint(GridPoint point) {
		return new GridPoint(-point.getX(), -point.getY());
	}

	public static NdPoint sumPoints(NdPoint... points) {
		double x = 0, y = 0/* , z = 0 */;
		for (NdPoint point : points) {
			x += point.getX();
			y += point.getY();
			// z += point.getZ();
		}
		return new NdPoint(x, y/* , z */);
	}

	public static GridPoint sumPoints(GridPoint... points) {
		int x = 0, y = 0/* , z = 0 */;
		for (GridPoint point : points) {
			x += point.getX();
			y += point.getY();
			// z += point.getZ();
		}
		return new GridPoint(x, y/* , z */);
	}

	public static int[] getCoords(GridPoint point) {
		if (point != null)
			return new int[] { point.getX(), point.getY() /* , point.getZ() */};
		return new int[2];
	}

	public static double[] getCoords(NdPoint point) {
		if (point != null)
			return new double[] { point.getX(), point.getY() /* , point.getZ() */};
		return new double[2];
	}

	public static double vectorIntensity(NdPoint vector) {
		return Math.sqrt(Math.pow(vector.getX(), 2)
				+ Math.pow(vector.getY(), 2)) /* + Math.pow(vector.getZ(), 2)) */;
	}

	public static NdPoint[] calculateExplosionVel(double m1, double m2,
			NdPoint vi) {
		double xi = vi.getX();
		double yi = vi.getY();
		// double zi = vi.getZ();
		double x1, y1, x2, y2;
		x2 = (xi * (m1 + m2) * m2) / (Math.pow(m1 + m2, 2) + m1 * m2);
		y2 = (yi * (m1 + m2) * m2) / (Math.pow(m1 + m2, 2) + m1 * m2);
		// z2 = (zi * (m1 + m2) * m2) / (Math.pow(m1 + m2, 2) + m1 * m2);
		x1 = ((m1 + m2) * xi - m2 * x2) / m1;
		y1 = ((m1 + m2) * yi - m2 * y2) / m1;
		// z1 = ((m1 + m2) * zi - m2 * z2) / m1;
		return new NdPoint[] { new NdPoint(x1, y1), new NdPoint(x2, y2) };

	}
}
