package shapecalculus.behaviour;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

public class Choice extends NonAtomicBehaviour {

	public Choice(Behaviour... behaviours) throws BehaviourException {
		if (behaviours.length < 1)
			throw new BehaviourException(
					"Choice must have at least one operand.");
		this.operands = new ArrayList<Behaviour>();
		for (Behaviour b : behaviours) {
			if (b instanceof DistanceSensible) {
				DistanceSensible bsens = (DistanceSensible) b;
				int[] directions = bsens.getSurface().getDirection();
				if (directions.length > 1)
					this.operands.add(bsens
							.returnDistanceSensibleEquivalent(directions));
				else
					this.operands.add(b);
			} else
				this.operands.add(b);
		}
		this.currentIndex = -1;
	}

	private Choice(List<Behaviour> operands) {
		this.operands = new ArrayList<Behaviour>();
		for (Behaviour b : operands)
			this.operands.add((Behaviour) b.clone());

	}

	// @Override
	// public int perform() {
	// if (this.currentIndex == -2)
	// return 0;
	// int ret = -1;
	// if (this.currentIndex >= 0) {
	// ret = this.operands.get(this.currentIndex).perform();
	// if (ret < 1)
	// this.currentIndex = -2;
	// } else {
	// boolean nil = false;
	// for (int i = 0; i < this.operands.size(); i++)
	// if ((ret = this.operands.get(i).perform()) == 0)
	// nil = true;
	// else if (ret > 0) {
	// this.currentIndex = i;
	// break;
	// }
	// if (ret < 0 && nil)
	// ret = 0;
	// if (ret < 1)
	// this.currentIndex = -1;
	// }
	// return ret;
	// }

	@Override
	public int perform(Iterator<Integer> path) {
		if (this.currentIndex == -2 || !path.hasNext() || !isPathInitialized)
			return 0;
		int tempIndex = path.next();
		Behaviour tempBehaviour = this.operands.get(tempIndex);
		int ret = -1;
		ret = tempBehaviour.perform(path);
		if (!(tempBehaviour instanceof Delay)) {
			this.currentIndex = tempIndex;
			if (ret < 1)
				this.currentIndex = -2;
			return ret;
		}
		return 1;

	}

	@Override
	public Set<AtomicBehaviour> getEnabledActions() {
		Set<AtomicBehaviour> ret = new HashSet<AtomicBehaviour>();
		if (this.currentIndex == -2)
			return ret;

		if (this.currentIndex >= 0) {
			Behaviour b = this.operands.get(this.currentIndex);
			if (b instanceof AtomicBehaviour
					&& ((AtomicBehaviour) b).canPerform())
				ret.add((AtomicBehaviour) b);
			else if (b instanceof NonAtomicBehaviour)
				ret.addAll(((NonAtomicBehaviour) b).getEnabledActions());
		} else {
			for (Behaviour b : this.operands) {
				if (b instanceof AtomicBehaviour
						&& ((AtomicBehaviour) b).canPerform())
					ret.add((AtomicBehaviour) b);
				else if (b instanceof NonAtomicBehaviour)
					ret.addAll(((NonAtomicBehaviour) b).getEnabledActions());
			}
		}
		return ret;

	}

	@Override
	public Object clone() {
		Choice c = new Choice(this.operands);
		c.currentIndex = this.currentIndex;
		c.initPath(this.path);
		return c;
	}

}
