package shapecalculus.behaviour;

import java.util.Iterator;

import repast.simphony.context.Context;
import repast.simphony.space.grid.GridPoint;
import shapecalculus.BasicShape;
import shapecalculus.ShapeException;
import shapecalculus.Surface;
import shapecalculus.behaviour.channel.Channel;
import shapecalculus.behaviour.channel.InChannel;
import shapecalculus.behaviour.channel.OutChannel;

public class Consume extends DistanceSensible {

	public Consume(Channel channel) {
		this.channel = channel;
	}

	@Override
	public int perform(Iterator<Integer> path) {
		DistanceSensible otherAct = this.synchronizingAction;
		if (this.isOutput() && otherAct != null) {
			BasicShape otherShape = otherAct.getShape();
			Context c = otherShape.getContext();
			this.getShape().moveTo(
					otherAct.channel.getSurface().getAdjacentPoint(
							otherShape.getPosition()), false);
			Channel chan;
			for (int i = 0; i < 6; i++) {
				chan = otherShape.getSurfaceAt(i);
				if (chan != null) {
					chan.getBindedTo().removeBond();
					chan.removeBond();
				}
			}
			c.remove(otherShape);
		}
		return 0;
	}

	@Override
	public GridPoint getExtent() {
		if (this.channel instanceof OutChannel)
			return ((OutChannel) this.channel).getExtent();
		return new GridPoint(1, 1, 1);
	}

	@Override
	public boolean isCompatibleWith(DistanceSensible action) {
		return this.channel.canBindWith(action.getChannel())
				&& (action instanceof Consume);
	}

	@Override
	public Behaviour returnDistanceSensibleEquivalent(int[] directions) {
		Behaviour[] operands = new Consume[directions.length];
		for (int i = 0; i < directions.length; i++) {
			try {
				if (this.channel instanceof InChannel)
					operands[i] = new Consume(new InChannel(
							this.channel.getChannelName(),
							this.channel.getShape(),
							new Surface(directions[i]),
							this.channel.getPriority()));
				else if (this.channel instanceof OutChannel)
					operands[i] = new Bind(new OutChannel(
							this.channel.getChannelName(),
							this.channel.getShape(),
							new Surface(directions[i]),
							((OutChannel) this.channel).getExtent(),
							this.channel.getPriority()));
			} catch (ShapeException e) {
				e.printStackTrace();
			}
		}
		try {
			return new Choice(operands);
		} catch (BehaviourException e) {
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public boolean canPerform() {
		return this.channel.getBindedTo() == null
				&& this.channel.hasFreeSurface();
	}

	@Override
	public Object clone() {
		Consume c = new Consume(this.channel);
		c.initPath(this.path);
		return c;
	}

}
