package shapecalculus.behaviour;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

public class Sequence extends NonAtomicBehaviour {

	public Sequence(Behaviour... behaviours) throws BehaviourException {
		if (behaviours.length < 1)
			throw new BehaviourException(
					"Choice must have at least one operand.");
		this.operands = new ArrayList<Behaviour>();
		for (Behaviour b : behaviours) {
			if (b instanceof DistanceSensible) {
				DistanceSensible bsens = (DistanceSensible) b;
				int[] directions = bsens.getSurface().getDirection();
				if (directions.length > 1)
					this.operands.add(bsens
							.returnDistanceSensibleEquivalent(directions));
				else
					this.operands.add(b);
			} else
				this.operands.add(b);
		}
		this.currentIndex = 0;
	}

	private Sequence(List<Behaviour> operands) {
		this.operands = new ArrayList<Behaviour>();
		for (Behaviour b : operands)
			this.operands.add((Behaviour) b.clone());
	}

//	@Override
//	public int perform() {
//		if (!(currentIndex < this.operands.size()))
//			return 0;
//		int ret = this.operands.get(currentIndex).perform();
//		if (ret == 0) {
//			currentIndex++;
//			if (!(currentIndex < this.operands.size()))
//				ret = 0;
//			else
//				ret = this.operands.get(currentIndex).perform();
//		}
//		return ret;
//	}

	@Override
	public int perform(Iterator<Integer> path) {
		if (!(currentIndex < this.operands.size() && path.hasNext() && isPathInitialized))
			return 0;
		this.currentIndex = path.next();
		int ret = this.operands.get(currentIndex).perform(path);
		if (ret == 0) {
			currentIndex++;
			if (!(currentIndex < this.operands.size()))
				ret = 0;
			else
				ret = 1;
			// ret = this.operands.get(currentIndex).perform();
		}
		return ret;
	}

	public Set<AtomicBehaviour> getEnabledActions() {
		Set<AtomicBehaviour> ret = new HashSet<AtomicBehaviour>();
		if (!(currentIndex < this.operands.size()))
			return ret;

		Behaviour b = this.operands.get(currentIndex);

		if (b instanceof AtomicBehaviour && ((AtomicBehaviour) b).canPerform())
			ret.add((AtomicBehaviour) b);
		else if (b instanceof NonAtomicBehaviour)
			ret.addAll(((NonAtomicBehaviour) b).getEnabledActions());

		return ret;
	}

	public Object clone() {
		Sequence s = new Sequence(this.operands);
		s.currentIndex = this.currentIndex;
		s.initPath(this.path);
		return s;
	}

}
