package shapecalculus.behaviour.channel;

import shapecalculus.BasicShape;
import shapecalculus.ComplexShape;
import shapecalculus.Surface;

public abstract class Channel {

	protected BasicShape shape;
	protected Surface surface;
	protected Channel bindedTo;
	protected String channelName;
	protected double priority;

	public double getPriority() {
		return priority;
	}

	public void setPriority(double priority) {
		this.priority = priority;
	}

	public Channel getBindedTo() {
		return bindedTo;
	}

	public void setBindedTo(Channel bindedTo) {
		this.bindedTo = bindedTo;
		this.shape.setSurfaceAt(this, this.surface.getDirection()[0]);
	}

	public void removeBond() {
		this.bindedTo = null;
		this.shape.setSurfaceAt(null, this.surface.getDirection()[0]);
	}

	public BasicShape getShape() {
		return shape;
	}

	public Surface getSurface() {
		return surface;
	}

	public boolean hasFreeSurface() {
		return this.shape.getSurfaceAt(this.surface.getDirection()[0]) == null;
	}

	public abstract boolean isOutput();

	public String getChannelName() {
		return channelName;
	}

	public boolean canBindWith(Channel c) {
		boolean sameName = c.channelName.equals(this.channelName);
		boolean complementary = c.isOutput() ^ this.isOutput();
		boolean oppositeSurfaces = c.getSurface().isOppositeTo(
				this.getSurface());
		boolean differentShapes = c.shape != this.shape;
		ComplexShape complexThis = this.shape.getComplexShape();
		ComplexShape complexC = c.shape.getComplexShape();
		boolean differentComplexShapes = (complexThis == null)
				|| (complexC == null) || (complexC != complexThis);
		return sameName && complementary && oppositeSurfaces && differentShapes
				&& differentComplexShapes;
	}

	public boolean canSplitFrom(Channel c) {
		Channel thisBinded = this.getBindedEquivalent();
		Channel otherBinded = c.getBindedEquivalent();
		return thisBinded.getBindedTo() == otherBinded;
	}
	
	public Channel getBindedEquivalent(){
		return this.shape.getSurfaceAt(this.surface.getDirection()[0]);
	}
}
